@@ -19,6 +19,7 @@
  * 02110-1301, USA.
  */
 
+#include "alert.h"
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,29 +29,6 @@
 #include "print.h"
 #include "options.h"
 
-enum alert_t alert_option_parse(const char *arg)
-{
-    enum alert_t alert = option.alert; // Default
-
-    if (arg != NULL)
-    {
-        if (strcmp(arg, "none") == 0)
-        {
-            return ALERT_NONE;
-        }
-        else if (strcmp(arg, "bell") == 0)
-        {
-            return ALERT_BELL;
-        }
-        else if (strcmp(arg, "blink") == 0)
-        {
-            return ALERT_BLINK;
-        }
-    }
-
-    return alert;
-}
-
 void blink_background(void)
 {
     // Turn on reverse video
