@@ -45,9 +45,9 @@ int main(int argc, char *argv[])
     /* Parse command-line options (1st pass) */
     options_parse(argc, argv);
 
-    if (option.complete_sub_configs)
+    if (option.complete_profiles)
     {
-        config_file_show_sub_configurations();
+        config_file_show_profiles();
         return status;
     }
 
@@ -71,6 +71,9 @@ int main(int argc, char *argv[])
         interactive_mode = false;
     }
 
+    /* Switch error output format */
+    switch_error_output_mode();
+
     /* Configure output terminal */
     if (isatty(fileno(stdout)))
     {
@@ -121,8 +124,9 @@ int main(int argc, char *argv[])
     tty_input_thread_wait_ready();
 
     /* Connect to tty device */
-    if (option.no_autoconnect)
+    if (option.no_reconnect)
     {
+        tty_search();
         status = tty_connect();
     }
     else
@@ -131,7 +135,7 @@ int main(int argc, char *argv[])
         while (true)
         {
             tty_wait_for_device();
-            status = tty_connect();
+            tty_connect();
         }
     }
 
