@@ -26,12 +26,12 @@
 
 #define UNUSED(expr) do { (void)(expr); } while (0)
 
-char * current_time(void);
 void delay(long ms);
-long string_to_long(char *string);
 int ctrl_key_code(unsigned char key);
-void alert_connect(void);
-void alert_disconnect(void);
-bool fs_dir_exists(const char *path);
 bool regex_match(const char *string, const char *pattern);
+unsigned long djb2_hash(const unsigned char *str);
+char *base62_encode(unsigned long num);
 int read_poll(int fd, void *data, size_t len, int timeout);
+double get_current_time(void);
+bool match_patterns(const char *string, const char *patterns);
+int execute_shell_command(int fd, const char *command);
