@@ -77,6 +77,7 @@ extern char ansi_format[];
         fprintf (stdout, "\r[%s] Warning: " format "\r\n", timestamp_current_time(), ## args); \
         else \
         ansi_printf("[%s] Warning: " format, timestamp_current_time(), ## args); \
+        print_tainted = false; \
     } \
 }
 
@@ -86,10 +87,19 @@ extern char ansi_format[];
     { \
         if (print_tainted) \
         putchar('\n'); \
-        if (option.color < 0) \
-        fprintf (stdout, "\r[%s] Error: " format "\r\n", timestamp_current_time(), ## args); \
-        else \
-        ansi_printf("[%s] Error: " format, timestamp_current_time(), ## args); \
+        if (option.color < 0) { \
+            if (error_normal) \
+                fprintf (stderr, "Error: " format "\n", ## args); \
+            else \
+                fprintf (stderr, "\r[%s] Error: " format "\r\n", timestamp_current_time(), ## args); \
+        } \
+        else { \
+            if (error_normal) \
+            { ansi_error_printf("Error: " format, ## args); }\
+            else \
+            { ansi_error_printf("[%s] Error: " format, timestamp_current_time(), ## args); }\
+        } \
+        print_tainted = false; \
     } \
 }
 
