@@ -34,13 +34,28 @@
 #include "options.h"
 #include "tty.h"
 #include "xymodem.h"
+#include "log.h"
+#include "script.h"
+#include "fs.h"
 
 #define MAX_BUFFER_SIZE 2000 // Maximum size of circular buffer
 
 static int device_fd;
 static char circular_buffer[MAX_BUFFER_SIZE];
+static char match_string[MAX_BUFFER_SIZE];
 static int buffer_size = 0;
 
+static char script_init[] =
+"function set(arg)\n"
+"    local dtr = arg.DTR or -1\n"
+"    local rts = arg.RTS or -1\n"
+"    local cts = arg.CTS or -1\n"
+"    local dsr = arg.DSR or -1\n"
+"    local cd = arg.CD or -1\n"
+"    local ri = arg.RI or -1\n"
+"    line_set(dtr, rts, cts, dsr, cd, ri)\n"
+"end\n";
+
 // lua: sleep(seconds)
 static int sleep_(lua_State *L)
 {
@@ -75,87 +90,56 @@ static int msleep(lua_State *L)
     return 0;
 }
 
-// lua: high(line)
-static int high(lua_State *L)
+// lua: line_set(dtr,rts,cts,dsr,cd,ri)
+static int line_set(lua_State *L)
 {
-    long line = lua_tointeger(L, 1);
+    tty_line_config_t line_config[6] = { };
 
-    if (line < 0)
+    int dtr = lua_tointeger(L, 1);
+    int rts = lua_tointeger(L, 2);
+    int cts = lua_tointeger(L, 3);
+    int dsr = lua_tointeger(L, 4);
+    int cd = lua_tointeger(L, 5);
+    int ri = lua_tointeger(L, 6);
+
+    if (dtr != -1)
     {
-        return 0;
+        line_config[0].mask = TIOCM_DTR;
+        line_config[0].value = dtr;
+        line_config[0].reserved = true;
+    }
+    if (rts != -1)
+    {
+        line_config[1].mask = TIOCM_RTS;
+        line_config[1].value = rts;
+        line_config[1].reserved = true;
+    }
+    if (cts != -1)
+    {
+        line_config[2].mask = TIOCM_CTS;
+        line_config[2].value = cts;
+        line_config[2].reserved = true;
+    }
+    if (dsr != -1)
+    {
+        line_config[3].mask = TIOCM_DSR;
+        line_config[3].value = dsr;
+        line_config[3].reserved = true;
+    }
+    if (cd != -1)
+    {
+        line_config[4].mask = TIOCM_CD;
+        line_config[4].value = cd;
+        line_config[4].reserved = true;
+    }
+    if (ri != -1)
+    {
+        line_config[5].mask = TIOCM_RI;
+        line_config[5].value = ri;
+        line_config[5].reserved = true;
     }
 
-    tty_line_set(device_fd, line, LINE_HIGH);
-
-    return 0;
-}
-
-// lua: low(line)
-static int low(lua_State *L)
-{
-    long line = lua_tointeger(L, 1);
-
-    if (line < 0)
-    {
-        return 0;
-    }
-
-    tty_line_set(device_fd, line, LINE_LOW);
-
-    return 0;
-}
-
-// lua: toggle(line)
-static int toggle(lua_State *L)
-{
-    long line = lua_tointeger(L, 1);
-
-    if (line < 0)
-    {
-        return 0;
-    }
-
-    tty_line_toggle(device_fd, line);
-
-    return 0;
-}
-
-// lua: config_high(line)
-static int config_high(lua_State *L)
-{
-    long line = lua_tointeger(L, 1);
-
-    if (line < 0)
-    {
-        return 0;
-    }
-
-    tty_line_config(line, true);
-
-    return 0;
-}
-
-// lua: config_low(line)
-static int config_low(lua_State *L)
-{
-    long line = lua_tointeger(L, 1);
-
-    if (line < 0)
-    {
-        return 0;
-    }
-
-    tty_line_config(line, false);
-
-    return 0;
-}
-
-// lua: config_apply(line)
-static int config_apply(lua_State *L)
-{
-    UNUSED(L);
-
-    tty_line_config_apply();
+    tty_line_set(device_fd, line_config);
 
     return 0;
 }
@@ -204,18 +188,14 @@ static int send(lua_State *L)
     }
 
     ret = write(device_fd, string, strlen(string));
-    if (ret < 0)
-    {
-        tio_error_print("%s\n", strerror(errno));
-    }
 
     lua_pushnumber(L, ret);
 
     return 1;
 }
 
-// Function to add a character to the circular buffer
-void add_to_buffer(char c)
+// Function to add a character to the circular expect buffer
+static void expect_buffer_add(char c)
 {
     if (buffer_size < MAX_BUFFER_SIZE)
     {
@@ -229,18 +209,26 @@ void add_to_buffer(char c)
     }
 }
 
-// Function to match against the circular buffer using regex
-bool match_regex(regex_t *regex)
+// Function to match against the circular expect buffer using regex
+static bool match_regex(regex_t *regex)
 {
     char buffer[MAX_BUFFER_SIZE + 1]; // Temporary buffer for regex matching
+    const char *s = circular_buffer;
+    regmatch_t pmatch[1];
+    regoff_t len;
+
     memcpy(buffer, circular_buffer, buffer_size);
     buffer[buffer_size] = '\0'; // Null-terminate the buffer
 
     // Match against the regex
-    int ret = regexec(regex, buffer, 0, NULL, 0);
+    int ret = regexec(regex, buffer, 1, pmatch, 0);
     if (!ret)
     {
         // Match found
+        len = pmatch[0].rm_eo - pmatch[0].rm_so;
+        memcpy(match_string, s + pmatch[0].rm_so, len);
+        match_string[len] = '\0';
+
         return true;
     }
     else if (ret == REG_NOMATCH)
@@ -256,6 +244,59 @@ bool match_regex(regex_t *regex)
     return false;
 }
 
+// lua: ret,string = read_string(size, timeout)
+static int read_string(lua_State *L)
+{
+    int size = lua_tointeger(L, 1);
+    int timeout = lua_tointeger(L, 2);
+    int ret = 0;
+
+    char *buffer = malloc(size);
+    if (buffer == NULL)
+    {
+        ret = -1; // Error
+        goto error;
+    }
+
+    if (timeout == 0)
+    {
+        timeout = -1; // Wait forever
+    }
+
+    ssize_t bytes_read = read_poll(device_fd, buffer, size, timeout);
+    if (bytes_read < 0)
+    {
+        ret = -1; // Error
+        goto error;
+    }
+    else if (bytes_read == 0)
+    {
+        ret = 0; // Timeout
+        goto error;
+    }
+
+    for (ssize_t i=0; i<bytes_read; i++)
+    {
+        putchar(buffer[i]);
+
+        if (option.log)
+        {
+            log_putc(buffer[i]);
+        }
+    }
+
+    ret = bytes_read;
+
+error:
+    lua_pushnumber(L, ret);
+    if (buffer != NULL)
+    {
+        lua_pushstring(L, buffer);
+        free(buffer);
+    }
+    return 2;
+}
+
 // lua: expect(string, timeout)
 static int expect(lua_State *L)
 {
@@ -265,6 +306,10 @@ static int expect(lua_State *L)
     int ret = 0;
     char c;
 
+    // Resets buffer to ignore previous `expect` calls
+    buffer_size = 0;
+    match_string[0] = '\0';
+
     if ((string == NULL) || (timeout < 0))
     {
         ret = -1;
@@ -293,10 +338,17 @@ static int expect(lua_State *L)
         if (bytes_read > 0)
         {
             putchar(c);
-            add_to_buffer(c);
+            expect_buffer_add(c);
+
+            if (option.log)
+            {
+                log_putc(c);
+            }
+
             // Match against the entire buffer
             if (match_regex(&regex))
             {
+                ret = 1;
                 break;
             }
         }
@@ -312,7 +364,8 @@ static int expect(lua_State *L)
 
 error:
     lua_pushnumber(L, ret);
-    return 1;
+    lua_pushstring(L, match_string);
+    return 2;
 }
 
 // lua: exit(code)
@@ -325,6 +378,60 @@ static int exit_(lua_State *L)
     return 0;
 }
 
+// lua: list = tty_search()
+static int tty_search_(lua_State *L)
+{
+    UNUSED(L);
+    GList *iter;
+    int i = 1;
+
+    GList *device_list = tty_search_for_serial_devices();
+
+    if (device_list == NULL)
+    {
+        return 0;
+    }
+
+    // Create a new table
+    lua_newtable(L);
+
+    // Iterate through found devices
+    for (iter = device_list; iter != NULL; iter = g_list_next(iter))
+    {
+        device_t *device = (device_t *) iter->data;
+
+        // Create a new sub-table for each serial device
+        lua_newtable(L);
+
+        // Add elements to the table
+        lua_pushstring(L, "path");
+        lua_pushstring(L, device->path);
+        lua_settable(L, -3);
+
+        lua_pushstring(L, "tid");
+        lua_pushstring(L, device->tid);
+        lua_settable(L, -3);
+
+        lua_pushstring(L, "uptime");
+        lua_pushnumber(L, device->uptime);
+        lua_settable(L, -3);
+
+        lua_pushstring(L, "driver");
+        lua_pushstring(L, device->driver);
+        lua_settable(L, -3);
+
+        lua_pushstring(L, "description");
+        lua_pushstring(L, device->description);
+        lua_settable(L, -3);
+
+        // Set the sub-table as a row in the main table
+        lua_rawseti(L, -2, i++);
+    }
+
+    // Return table
+    return 1;
+}
+
 static void script_buffer_run(lua_State *L, const char *script_buffer)
 {
     int error;
@@ -334,7 +441,7 @@ static void script_buffer_run(lua_State *L, const char *script_buffer)
     if (error)
     {
         tio_warning_printf("lua: %s\n", lua_tostring(L, -1));
-        lua_pop(L, 1);  /* pop error message from the stack */
+        lua_pop(L, 1);  /* Pop error message from the stack */
     }
 }
 
@@ -342,16 +449,13 @@ static const struct luaL_Reg tio_lib[] =
 {
     { "sleep", sleep_},
     { "msleep", msleep},
-    { "high", high},
-    { "low", low},
-    { "toggle", toggle},
-    { "config_high", config_high},
-    { "config_low", config_low},
-    { "config_apply", config_apply},
+    { "line_set", line_set},
     { "modem_send", modem_send},
     { "send", send},
+    { "read", read_string},
     { "expect", expect},
     { "exit", exit_},
+    { "tty_search", tty_search_},
     {NULL, NULL}
 };
 
@@ -374,6 +478,18 @@ static void luaL_setfuncs (lua_State *L, const luaL_Reg *l, int nup)
 }
 #endif
 
+static void script_load(lua_State *L)
+{
+    int error;
+
+    error = luaL_loadbuffer(L, script_init, strlen(script_init), "tio") || lua_pcall(L, 0, 0, 0);
+    if (error)
+    {
+        tio_error_print("%s\n", lua_tostring(L, -1));
+        lua_pop(L, 1); // Pop error message from the stack
+    }
+}
+
 int lua_register_tio(lua_State *L)
 {
     // Register lxi functions
@@ -381,6 +497,9 @@ int lua_register_tio(lua_State *L)
     luaL_setfuncs(L, tio_lib, 0);
     lua_pop(L, 1);
 
+    // Load lua init script
+    script_load(L);
+
     return 0;
 }
 
@@ -408,12 +527,9 @@ void script_set_global(lua_State *L, const char *name, long value)
 
 void script_set_globals(lua_State *L)
 {
-    script_set_global(L, "DTR", TIOCM_DTR);
-    script_set_global(L, "RTS", TIOCM_RTS);
-    script_set_global(L, "CTS", TIOCM_CTS);
-    script_set_global(L, "DSR", TIOCM_DSR);
-    script_set_global(L, "CD", TIOCM_CD);
-    script_set_global(L, "RI", TIOCM_RI);
+    script_set_global(L, "toggle", 2);
+    script_set_global(L, "high", 1);
+    script_set_global(L, "low", 0);
     script_set_global(L, "XMODEM_CRC", XMODEM_CRC);
     script_set_global(L, "XMODEM_1K", XMODEM_1K);
     script_set_global(L, "YMODEM", YMODEM);
@@ -447,3 +563,18 @@ void script_run(int fd)
 
     lua_close(L);
 }
+
+const char *script_run_state_to_string(script_run_t state)
+{
+    switch (state)
+    {
+        case SCRIPT_RUN_ONCE:
+            return "once";
+        case SCRIPT_RUN_ALWAYS:
+            return "always";
+        case SCRIPT_RUN_NEVER:
+            return "never";
+        default:
+            return "Unknown";
+    }
+}
