@@ -93,60 +93,3 @@ char *timestamp_current_time(void)
 
     return (len < TIME_STRING_SIZE_MAX) ? time_string : NULL;
 }
-
-const char* timestamp_state_to_string(enum timestamp_t timestamp)
-{
-    switch (timestamp)
-    {
-        case TIMESTAMP_NONE:
-            return "disabled";
-            break;
-
-        case TIMESTAMP_24HOUR:
-            return "24hour";
-            break;
-
-        case TIMESTAMP_24HOUR_START:
-            return "24hour-start";
-            break;
-
-        case TIMESTAMP_24HOUR_DELTA:
-            return "24hour-delta";
-            break;
-
-        case TIMESTAMP_ISO8601:
-            return "iso8601";
-            break;
-
-        default:
-            return "unknown";
-            break;
-    }
-}
-
-enum timestamp_t timestamp_option_parse(const char *arg)
-{
-    enum timestamp_t timestamp = TIMESTAMP_24HOUR; // Default
-
-    if (arg != NULL)
-    {
-        if (strcmp(arg, "24hour") == 0)
-        {
-            return TIMESTAMP_24HOUR;
-        }
-        else if (strcmp(arg, "24hour-start") == 0)
-        {
-            return TIMESTAMP_24HOUR_START;
-        }
-        else if (strcmp(arg, "24hour-delta") == 0)
-        {
-            return TIMESTAMP_24HOUR_DELTA;
-        }
-        else if (strcmp(arg, "iso8601") == 0)
-        {
-            return TIMESTAMP_ISO8601;
-        }
-    }
-
-    return timestamp;
-}
